import ephem
from ephem import *
import matplotlib.pyplot as plt
import datetime as dt
import numpy as np
from statistics import mean#calcul de la moyenne des longitudes (planètes gazeuses)

plt.rcParams["font.family"] = "Ubuntu"#ou autre police installée sur l'ordinateur

#Dates limitant la période souhaitée :---------------------------------------------------------
date_debut=dt.datetime(2022, 10, 1)
date_fin=dt.datetime(2023, 2, 1)
pasJours=3#intervalle entre deux dates
pas=dt.timedelta(days=pasJours)
#-------------------------------------------------------------------
#Création d'une liste de dates :
epoque=[]
date=date_debut
while date<date_fin:
    epoque.append(date)
    date=date+pas

#Liste des couleurs par planète :
couleurs={
'Mercure':'gray',
'Vénus':'tan',
'Terre':'royalblue', 
'Mars':'orangered',
'Jupiter':'orange',
'Saturne':'goldenrod', 
'Uranus' : 'skyblue', 
'Neptune' : 'mediumblue'
}

#%%%%%%%%%%%%%%%%%%% Fonctions
def coordonneesplanete(planete, periode):#calcule et renvoie listes de longitude et latitude
    longitude, distance=[], []
    for jour in periode:
        planete.compute(str(jour))
        longitude.append(float(planete.hlong))
        distance.append(planete.sun_distance)
    return longitude, distance

def angleComplem(serie):
    for n,  angle in enumerate(serie):
        if angle >np.pi:
            angle=angle-2*np.pi
        serie[n]=angle
    return serie
    
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#Paramètres du graphique :
#fig=plt.figure(figsize=(21/2.54, 29.7/2.54 ), tight_layout=True)#dimensions A4
fig=plt.figure(figsize=(10, 10), tight_layout=True)
ax=plt.subplot(projection='polar')
plt.title("Position héliocentrique des planètes", pad=20, fontsize=16)
plt.scatter(0, 0, s=200, c='orange', zorder=2)    #Soleil
taillePoint=40#taille du point des planètes sur le tracé

#plt.yticks([], [])#suppression des cercles et graduations en rayon.
plt.yticks([1], [])#conserver un cercle de rayon 1 UA.
#plt.yticks(np.arange(0, 2, 0.5), [])#graduer tous les 0.5 UA,sans chiffraison
plt.xticks(np.radians(np.arange(0, 360, 15)))#lignes d'angles, tous les 15°

#----------------------------------------------------------------------------
# Calcul des coordonnées de la Terre, et tracé de trajectoire :
soleil=Sun()
longitudeTerre, distanceTerre=[], []
for jour in epoque:
    soleil.compute(str(jour))
    longitudeTerre.append(float(soleil.hlong))#en calculant la longitude héliocentrique du Soleil, on a en fait celle de la Terre...
    distanceTerre.append(soleil.earth_distance)
ax.scatter(longitudeTerre, distanceTerre, s=taillePoint, ec='k', fc=couleurs['Terre'], label='Terre', zorder=3)
ax.scatter(longitudeTerre[0], distanceTerre[0], s=taillePoint, c='k')#1e position en noir

#----------------------------------------------------------------------------
#Autres planètes :
#mercure=ephem.Mercury()
#longitude, distance=coordonneesplanete(mercure, epoque)
#ax.scatter(longitude, distance, s=taillePoint,  ec='k', fc=couleurs['Mercure'], label='Mercure', zorder=3)
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir
#
#venus=ephem.Venus()
#longitude, distance=coordonneesplanete(venus, epoque)
#ax.scatter(longitude, distance, s=taillePoint,  ec='k', fc=couleurs['Vénus'], label='Vénus', zorder=3)
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir

mars=ephem.Mars()
longitude, distance=coordonneesplanete(mars, epoque)
ax.scatter(longitude, distance, s=taillePoint, ec='k', fc=couleurs['Mars'], label='Mars', zorder=3)
ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir

#----------------------------------------------------------------------------
#Planètes gazeuses :
"""
Tracé d'une flèche indiquant la longitude moyenne de Jupiter sur la période considérée.
Cette représentation évite des problèmes d'échelle pour les planètes très lointaines.
En supposant que l'époque considérée soit suffisamment courte pour que la longitude de ces planetes
varie très peu.
Attention : si durant l'époque considérée, la longitude de la planète prend des valeurs de part et d'autre de 0°,
la discontinuité de la longitude va donner une moyenne fantaisiste.
La fonction AngleComplémentaire() permet de contourner le problème, le cas échéant, mais seulement en
le reportant à plus tard !
#"""
"""
jupiter=ephem.Jupiter()
longitude, distance=coordonneesplanete(jupiter, epoque)
#ax.scatter(longitude, distance, s=taillePoint, ec='k', fc=couleurs['Jupiter'], label='Jupiter')
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir

ax.annotate("", xy=(mean(longitude), 0.1*mean(distance)), xytext=(0, 0),arrowprops=dict(arrowstyle="->", color=couleurs['Jupiter']), zorder=3)
plt.text(mean(longitude), 0.1*mean(distance), 'Jupiter', ha='left',va='center', rotation=np.degrees(mean(longitude)))

saturne=ephem.Saturn()
longitude, distance=coordonneesplanete(saturne, epoque)
#ax.scatter(longitude, distance, s=taillePoint, ec='k', fc=couleurs['Saturne'], label='Saturne')
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir
#Tracé d'une flèche indiquant la longitude moyenne de Saturne sur la période considérée :
ax.annotate("", xy=(mean(longitude), 0.1*mean(distance)), xytext=(0, 0),arrowprops=dict(arrowstyle="->", color=couleurs['Saturne']), zorder=3)
plt.text(mean(longitude), 0.1*mean(distance), 'Saturne', ha='left',va='center', rotation=np.degrees(mean(longitude)))

""
uranus=ephem.Uranus()
longitude, distance=coordonneesplanete(uranus, epoque)
#ax.scatter(longitude, distance, s=taillePoint, ec='k', fc=couleurs['Uranus'], label='Uranus')
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir
#Tracé d'une flèche indiquant la longitude moyenne sur la période considérée :
ax.annotate("", xy=(mean(longitude), 0.05*mean(distance)), xytext=(0, 0),arrowprops=dict(arrowstyle="->", color=couleurs['Uranus']), zorder=3)
plt.text(mean(longitude), 0.05*mean(distance), 'Uranus', ha='left',va='center', rotation=np.degrees(mean(longitude)))

neptune=ephem.Neptune()
longitude, distance=coordonneesplanete(neptune, epoque)
#ax.scatter(longitude, distance, s=taillePoint, ec='k', fc=couleurs['Neptune'], label='Neptune')
#ax.scatter(longitude[0], distance[0], s=taillePoint, c='k')#1e position en noir
#Tracé d'une flèche indiquant la longitude moyenne sur la période considérée :
ax.annotate("", xy=(mean(longitude), 0.05*mean(distance)), xytext=(0, 0),arrowprops=dict(arrowstyle="->", color=couleurs['Neptune']), zorder=3)
plt.text(mean(longitude), 0.05*mean(distance), 'Neptune', ha='left',va='center', rotation=np.degrees(mean(longitude)))
"""
#------------------------------------------------------------
#Les lignes suivantes tracent des traits entre la Terre et Mars, pour expliquer la rétrogradation, au voisinage des oppositions (décembre 2022, par exemple)
#longitudeMars, distanceMars=coordonneesplanete(mars, epoque)
#for N in range(len(longitudeTerre)):
#    plt.plot((longitudeTerre[N], longitudeMars[N]), (distanceTerre[N], distanceMars[N]), 'k')
#Fin du tracé des traits Terre-Mars---------------------------------------------------

#Affichage des dates :
texte="Période : \n du " + epoque[0].strftime("%d %b %Y") +"\n au " +epoque[-1].strftime("%d %b %Y") +"\n tous les %.i jours."%(pasJours)
plt.text(0.05, 0.90, texte, transform=fig.transFigure, c='white', va='top', bbox=dict(fc='firebrick'))

plt.legend()
fig.savefig("PositionHelioPlanetes.png", dpi=250)
fig.savefig("PositionHelioPlanetes.pdf")
plt.show()
